//[kaspresso](../../index.md)/[io.reactivex.exceptions](../index.md)/[ExtCompositeException](index.md)



# ExtCompositeException  
 [androidJvm] 

Represents an exception that is a composite of one or more other exceptions. A {@code ExtCompositeException} does not modify the structure of any exception it wraps, but at print-time it iterates through the list of Throwables contained in the composite in order to print them all. Its invariant is to contain an immutable, ordered (by insertion order), unique list of non-composite exceptions. You can retrieve individual exceptions in this list with getExceptions . The [printStackTrace](print-stack-trace.md) implementation handles the StackTrace in a customized way instead of using {@code getCause()} so that it can avoid circular references. If you invoke getCause , it will lazily create the causal chain but will stop if it finds any Throwable in the chain that it has already seen.

class [ExtCompositeException](index.md) : [RuntimeException](https://docs.oracle.com/javase/8/docs/api/java/lang/RuntimeException.html)   


## Constructors  
  
|  Name|  Summary| 
|---|---|
| [ExtCompositeException](-ext-composite-exception.md)|  [androidJvm] <br><br>Constructs a ExtCompositeException with the given array of Throwables as the list of suppressed exceptions.<br><br>open fun [ExtCompositeException](-ext-composite-exception.md)(exceptions: [Array](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-array/index.html)<[Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)>)   <br>
| [ExtCompositeException](-ext-composite-exception.md)|  [androidJvm] <br><br>Constructs a ExtCompositeException with the given array of Throwables as the list of suppressed exceptions.<br><br>open fun [ExtCompositeException](-ext-composite-exception.md)(errors: [Iterable](https://docs.oracle.com/javase/8/docs/api/java/lang/Iterable.html)<Out [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)>)   <br>


## Types  
  
|  Name|  Summary| 
|---|---|
| [CompositeExceptionCausalChain](-composite-exception-causal-chain/index.md)| [androidJvm]  <br>Content  <br>class [CompositeExceptionCausalChain](-composite-exception-causal-chain/index.md) : [RuntimeException](https://docs.oracle.com/javase/8/docs/api/java/lang/RuntimeException.html)  <br><br><br>
| [PrintStreamOrWriter](-print-stream-or-writer/index.md)| [androidJvm]  <br>Content  <br>abstract class [PrintStreamOrWriter](-print-stream-or-writer/index.md)  <br><br><br>
| [WrappedPrintStream](-wrapped-print-stream/index.md)| [androidJvm]  <br>Brief description  <br><br><br>Same abstraction and implementation as in JDK to allow PrintStream and PrintWriter to share implementation.<br><br>  <br>Content  <br>class [WrappedPrintStream](-wrapped-print-stream/index.md) : [ExtCompositeException.PrintStreamOrWriter](-print-stream-or-writer/index.md)  <br><br><br>
| [WrappedPrintWriter](-wrapped-print-writer/index.md)| [androidJvm]  <br>Content  <br>class [WrappedPrintWriter](-wrapped-print-writer/index.md) : [ExtCompositeException.PrintStreamOrWriter](-print-stream-or-writer/index.md)  <br><br><br>


## Functions  
  
|  Name|  Summary| 
|---|---|
| [addSuppressed](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#addSuppressed-java.lang.Throwable-)| [androidJvm]  <br>Content  <br>fun [addSuppressed](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#addSuppressed-java.lang.Throwable-)(p: [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html))  <br><br><br>
| [fillInStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#fillInStackTrace--)| [androidJvm]  <br>Content  <br>open fun [fillInStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#fillInStackTrace--)(): [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)  <br><br><br>
| [getCause](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getCause--)| [androidJvm]  <br>Content  <br>open fun [getCause](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getCause--)(): [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)  <br><br><br>
| [getLocalizedMessage](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getLocalizedMessage--)| [androidJvm]  <br>Content  <br>open fun [getLocalizedMessage](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getLocalizedMessage--)(): [String](https://docs.oracle.com/javase/8/docs/api/java/lang/String.html)  <br><br><br>
| [getMessage](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getMessage--)| [androidJvm]  <br>Content  <br>open fun [getMessage](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getMessage--)(): [String](https://docs.oracle.com/javase/8/docs/api/java/lang/String.html)  <br><br><br>
| [getOurStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getOurStackTrace--)| [androidJvm]  <br>Content  <br>open fun [getOurStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getOurStackTrace--)(): [Array](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-array/index.html)<[StackTraceElement](https://docs.oracle.com/javase/8/docs/api/java/lang/StackTraceElement.html)>  <br><br><br>
| [getRootCause](get-root-cause.md)| [androidJvm]  <br>Brief description  <br><br><br>Returns the root cause of {@code e} . If {@code e.getCause()} returns {@code null} or {@code e} , just return {@code e} itself.<br><br>  <br>Content  <br>open fun [getRootCause](get-root-cause.md)(e: [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)): [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)  <br><br><br>
| [getStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getStackTrace--)| [androidJvm]  <br>Content  <br>open fun [getStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getStackTrace--)(): [Array](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-array/index.html)<[StackTraceElement](https://docs.oracle.com/javase/8/docs/api/java/lang/StackTraceElement.html)>  <br><br><br>
| [getStackTraceDepth](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getStackTraceDepth--)| [androidJvm]  <br>Content  <br>open fun [getStackTraceDepth](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getStackTraceDepth--)(): [Int](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-int/index.html)  <br><br><br>
| [getStackTraceElement](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getStackTraceElement-int-)| [androidJvm]  <br>Content  <br>open fun [getStackTraceElement](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getStackTraceElement-int-)(p: [Int](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-int/index.html)): [StackTraceElement](https://docs.oracle.com/javase/8/docs/api/java/lang/StackTraceElement.html)  <br><br><br>
| [getSuppressed](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getSuppressed--)| [androidJvm]  <br>Content  <br>fun [getSuppressed](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#getSuppressed--)(): [Array](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-array/index.html)<[Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)>  <br><br><br>
| [initCause](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#initCause-java.lang.Throwable-)| [androidJvm]  <br>Content  <br>open fun [initCause](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#initCause-java.lang.Throwable-)(p: [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)): [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)  <br><br><br>
| [printEnclosedStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#printEnclosedStackTrace-java.lang.Throwable.PrintStreamOrWriter-java.lang.StackTraceElement[]-java.lang.String-java.lang.String-java.util.Set&lt;java.lang.Throwable&gt;-)| [androidJvm]  <br>Content  <br>open fun [printEnclosedStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#printEnclosedStackTrace-java.lang.Throwable.PrintStreamOrWriter-java.lang.StackTraceElement[]-java.lang.String-java.lang.String-java.util.Set&lt;java.lang.Throwable&gt;-)(p: [Throwable.PrintStreamOrWriter](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.PrintStreamOrWriter.html), p1: [Array](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-array/index.html)<[StackTraceElement](https://docs.oracle.com/javase/8/docs/api/java/lang/StackTraceElement.html)>, p2: [String](https://docs.oracle.com/javase/8/docs/api/java/lang/String.html), p3: [String](https://docs.oracle.com/javase/8/docs/api/java/lang/String.html), p4: [Set](https://docs.oracle.com/javase/8/docs/api/java/util/Set.html)<[Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)>)  <br><br><br>
| [printStackTrace](print-stack-trace.md)| [androidJvm]  <br>Brief description  <br><br><br><br><br>All of the following {@code printStackTrace} functionality is derived from JDK [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html){@code printStackTrace} . In particular, the {@code PrintStreamOrWriter} abstraction is copied wholesale. Changes from the official JDK implementation:<br><br><ul><li>no infinite loop detection</li><li>smaller critical section holding [PrintStream](https://docs.oracle.com/javase/8/docs/api/java/io/PrintStream.html) lock</li><li>explicit knowledge about the exceptions [List](https://docs.oracle.com/javase/8/docs/api/java/util/List.html) that this loops through</li></ul><br><br>  <br>Content  <br>open fun [printStackTrace](print-stack-trace.md)()  <br><br><br>[androidJvm]  <br>Content  <br>open fun [printStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#printStackTrace--)()  <br>open fun [printStackTrace](print-stack-trace.md)(s: [PrintStream](https://docs.oracle.com/javase/8/docs/api/java/io/PrintStream.html))  <br>open fun [printStackTrace](print-stack-trace.md)(s: [PrintWriter](https://docs.oracle.com/javase/8/docs/api/java/io/PrintWriter.html))  <br><br><br>
| [readObject](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#readObject-java.io.ObjectInputStream-)| [androidJvm]  <br>Content  <br>open fun [readObject](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#readObject-java.io.ObjectInputStream-)(p: [ObjectInputStream](https://docs.oracle.com/javase/8/docs/api/java/io/ObjectInputStream.html))  <br><br><br>
| [setStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#setStackTrace-java.lang.StackTraceElement[]-)| [androidJvm]  <br>Content  <br>open fun [setStackTrace](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#setStackTrace-java.lang.StackTraceElement[]-)(p: [Array](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-array/index.html)<[StackTraceElement](https://docs.oracle.com/javase/8/docs/api/java/lang/StackTraceElement.html)>)  <br><br><br>
| [size](size.md)| [androidJvm]  <br>Brief description  <br><br><br>Returns the number of suppressed exceptions.<br><br>  <br>Content  <br>open fun [size](size.md)(): [Int](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-int/index.html)  <br><br><br>
| [toString](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#toString--)| [androidJvm]  <br>Content  <br>open fun [toString](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#toString--)(): [String](https://docs.oracle.com/javase/8/docs/api/java/lang/String.html)  <br><br><br>
| [validateSuppressedExceptionsList](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#validateSuppressedExceptionsList-java.util.List&lt;java.lang.Throwable&gt;-)| [androidJvm]  <br>Content  <br>open fun [validateSuppressedExceptionsList](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#validateSuppressedExceptionsList-java.util.List&lt;java.lang.Throwable&gt;-)(p: [List](https://docs.oracle.com/javase/8/docs/api/java/util/List.html)<[Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)>): [Int](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-int/index.html)  <br><br><br>
| [writeObject](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#writeObject-java.io.ObjectOutputStream-)| [androidJvm]  <br>Content  <br>open fun [writeObject](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html#writeObject-java.io.ObjectOutputStream-)(p: [ObjectOutputStream](https://docs.oracle.com/javase/8/docs/api/java/io/ObjectOutputStream.html))  <br><br><br>


## Properties  
  
|  Name|  Summary| 
|---|---|
| [cause](index.md#io.reactivex.exceptions/ExtCompositeException/cause/#/PointingToDeclaration/)|  [androidJvm] private open val [cause](index.md#io.reactivex.exceptions/ExtCompositeException/cause/#/PointingToDeclaration/): [Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)   <br>
| [exceptions](index.md#io.reactivex.exceptions/ExtCompositeException/exceptions/#/PointingToDeclaration/)|  [androidJvm] private val [exceptions](index.md#io.reactivex.exceptions/ExtCompositeException/exceptions/#/PointingToDeclaration/): [List](https://docs.oracle.com/javase/8/docs/api/java/util/List.html)<[Throwable](https://docs.oracle.com/javase/8/docs/api/java/lang/Throwable.html)>   <br>
| [message](index.md#io.reactivex.exceptions/ExtCompositeException/message/#/PointingToDeclaration/)|  [androidJvm] private val [message](index.md#io.reactivex.exceptions/ExtCompositeException/message/#/PointingToDeclaration/): [String](https://docs.oracle.com/javase/8/docs/api/java/lang/String.html)   <br>

